package com.pravera.activity_rec_flutter.service

import DatabaseHelper
import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.CountDownTimer
import androidx.core.app.JobIntentService
import com.google.android.gms.location.ActivityRecognitionResult
import com.google.android.gms.location.DetectedActivity
import com.google.gson.Gson
import com.pravera.activity_rec_flutter.Constants
import com.pravera.activity_rec_flutter.errors.ErrorCodes
import com.pravera.activity_rec_flutter.models.ActivityData
import com.pravera.activity_rec_flutter.utils.ActivityRecognitionUtils
import com.pravera.activity_rec_flutter.utils.SharedPreferenceManager

class ActivityRecognitionIntentService : JobIntentService() {

    var currentActivity: ActivityData? = null
    var currentTime: Long = 0
    lateinit var mPrefs: SharedPreferenceManager


    companion object {
        val jsonConverter: Gson = Gson()
        fun enqueueWork(context: Context, intent: Intent) {
            enqueueWork(
                context, ActivityRecognitionIntentService::class.java,
                Constants.ACTIVITY_RECOGNITION_INTENT_SERVICE_JOB_ID, intent
            )
        }
    }

    override fun onHandleWork(intent: Intent) {
        val extractedResult = ActivityRecognitionResult.extractResult(intent)
        val probableActivities = extractedResult?.probableActivities
        val mostProbableActivity = probableActivities?.maxByOrNull { it.confidence } ?: return

        mPrefs = SharedPreferenceManager(this)

        val activityData = ActivityData(
            ActivityRecognitionUtils.getActivityTypeFromValue(mostProbableActivity.type),
            ActivityRecognitionUtils.getActivityConfidenceFromValue(mostProbableActivity.confidence)
        )

        Log.d("ActivityRecognitionIntentService","activity type =====> ${activityData.type}")

        if (currentActivity == null) {
            Log.d("ActivityRecognitionIntentService"," =====> currentActivity == null")
            currentTime = System.currentTimeMillis()
            currentActivity = activityData
        } else {
            Log.d("ActivityRecognitionIntentService"," =====> currentActivity != null")
            if (activityData != currentActivity) {
                Log.d("ActivityRecognitionIntentService"," =====> activityData != currentActivity")
                val timeDifferenceMillis = System.currentTimeMillis() - currentTime

                // Convert milliseconds to minutes
                val timeDifferenceMinutes = timeDifferenceMillis / 60000
                currentTime = System.currentTimeMillis()
                currentActivity = activityData

                when (activityData.type) {
                    "IN_VEHICLE" -> {
                        val points = timeDifferenceMinutes * 10 // per minute 10 points for vehicle
                        val storedPoints = mPrefs.getInt("_prefCurrentInVehiclePoints", 0)
                        mPrefs.putInt("_prefCurrentInVehiclePoints", storedPoints + points.toInt())
                    }

                    "ON_BICYCLE" -> {
                        val points = timeDifferenceMinutes * 8 // per minute 8 points for bycicle
                        val storedPoints = mPrefs.getInt("_prefCurrentOnByciclePoints", 0)
                        mPrefs.putInt("_prefCurrentOnByciclePoints", storedPoints + points.toInt())
                    }

                    "RUNNING" -> {
                        val points = timeDifferenceMinutes * 6 // per minute 6 points for running
                        val storedPoints = mPrefs.getInt("_prefCurrentRunningPoints", 0)
                        mPrefs.putInt("_prefCurrentRunningPoints", storedPoints + points.toInt())
                    }

                    "WALKING" -> {
                        val points = timeDifferenceMinutes * 4 // per minute 4 points for walking
                        val storedPoints = mPrefs.getInt("_prefCurrentWalkingPoints", 0)
                        mPrefs.putInt("_prefCurrentWalkingPoints", storedPoints + points.toInt())
                    }
                    "STILL" -> {
                        val points = timeDifferenceMinutes * 2 // per minute 2 points for still
                        val storedPoints = mPrefs.getInt("_prefCurrentStillPoints", 0)
                        mPrefs.putInt("_prefCurrentStillPoints", storedPoints + points.toInt())
                    }

                    else -> println("UNKNOWN activity")
                }
            }
        }


        val dbHelper = DatabaseHelper(this)
        dbHelper.addRecord(activityData.type, activityData.confidence, System.currentTimeMillis().toString())
        //dbHelper.getRecords()


        var prefsKey: String
        var prefsValue: String
        try {
            prefsKey = Constants.ACTIVITY_DATA_PREFS_KEY
            prefsValue = jsonConverter.toJson(activityData)
        } catch (e: Exception) {
            prefsKey = Constants.ACTIVITY_ERROR_PREFS_KEY
            prefsValue = ErrorCodes.ACTIVITY_DATA_ENCODING_FAILED.toString()
        }

        val prefs = getSharedPreferences(
            Constants.ACTIVITY_RECOGNITION_RESULT_PREFS_NAME, Context.MODE_PRIVATE
        ) ?: return
        with(prefs.edit()) {
            putString(prefsKey, prefsValue)
            commit()
        }
    }
}
